use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, sqlx::Type)]
pub struct Hersteller {
    /// Die Datenbank-ID
    pub id: i32,

    /// Der Name eines Herstellers
    name: String,
}

#[derive(InputObject, Debug)]
pub struct HerstellerCreateInput {
    /// Der Name eines Herstellers
    pub name: String,
}

#[derive(InputObject, Debug)]
pub struct HerstellerUpdateInput {
    /// Die Datenbank-ID
    pub id: i32,

    /// Der Name eines Herstellers
    pub name: Option<String>,
}

impl Hersteller {
    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Hersteller> {
        let row = sqlx::query_as!(Hersteller, "SELECT * FROM hersteller WHERE id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Hersteller>> {
        let rows = sqlx::query_as!(Hersteller, "SELECT id, name FROM hersteller")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn create(pool: &PgPool, input: &HerstellerCreateInput) -> Result<Hersteller> {
        let row = sqlx::query!(
            "INSERT INTO hersteller(name) VALUES ($1) RETURNING id",
            input.name
        )
        .fetch_one(pool)
        .await?;

        let result = Self::read_one(pool, &row.id).await?;
        Ok(result)
    }

    pub async fn update(pool: &PgPool, input: &HerstellerUpdateInput) -> Result<Hersteller> {
        sqlx::query!(
            "UPDATE hersteller SET name=$1 WHERE id = $2",
            input.name,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Hersteller::read_one(pool, &input.id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM hersteller WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
