use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

use super::{hersteller::Hersteller, typ::Typ};

#[derive(SimpleObject, FromRow, Deserialize, Serialize, Debug)]
pub struct Modell {
    /// Die Datenbank-ID
    pub id: i32,

    /// Der Geräte-Typ z.B. Monitor
    typ: Typ,

    /// Der Geräte-Hersteller z.B. Dell
    hersteller: Hersteller,

    /// Der Name eines Modells
    name: String,
}

#[derive(InputObject, Debug)]
pub struct ModellCreateInput {
    pub name: String,
    pub typ_id: i32,
    pub hersteller_id: i32,
}

#[derive(InputObject, Debug)]
pub struct ModellUpdateInput {
    pub id: i32,
    pub name: Option<String>,
    pub typ_id: Option<i32>,
    pub hersteller_id: Option<i32>,
}

impl Modell {
    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Modell> {
        let row = sqlx::query_as!(
            Modell,
            r#"
             select
               m.id, m.name,
               (t.id, t.name) as "typ!: Typ",
               (h.id, h.name) as "hersteller!: Hersteller"
             from modelle as m
             join typen as t on t.id = m.typ_id
             join hersteller as h on h.id = m.hersteller_id
             where m.id = $1;
         "#,
            id
        )
        .fetch_one(pool)
        .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Modell>> {
        let row = sqlx::query_as!(
            Modell,
            r#"
             select
                m.id, m.name,
                (t.id, t.name) as "typ!: Typ",
                (h.id, h.name) as "hersteller!: Hersteller"
             from modelle as m
             join typen as t on t.id = m.typ_id
             join hersteller as h on h.id = m.hersteller_id
         "#
        )
        .fetch_all(pool)
        .await?;

        Ok(row)
    }

    pub async fn create(pool: &PgPool, input: &ModellCreateInput) -> Result<Modell> {
        let row = sqlx::query!(
            r#"INSERT INTO modelle(typ_id, hersteller_id, name) VALUES ($1, $2, $3) RETURNING id"#,
            input.typ_id,
            input.hersteller_id,
            input.name
        )
        .fetch_one(pool)
        .await?;

        Self::read_one(pool, &row.id).await
    }

    pub async fn update(pool: &PgPool, input: &ModellUpdateInput) -> Result<Modell> {
        sqlx::query!(
            "UPDATE modelle SET name=COALESCE($1, name), typ_id=COALESCE($2, typ_id), hersteller_id=COALESCE($3, hersteller_id) WHERE id = $4",
            input.name,
            input.typ_id,
            input.hersteller_id,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Modell::read_one(pool, &input.id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM modelle WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
